#!/usr/bin/perl

package Mail::SpamAssassin::Plugin::URIIP;

use Mail::SpamAssassin::Plugin;
use Net::DNS;
use strict;
use bytes;

use vars qw(@ISA);
@ISA = qw(Mail::SpamAssassin::Plugin);

# constructor: register the eval rule
sub new {
	my $class = shift;
	my $mailsaobject = shift;

	# some boilerplate...
	$class = ref($class) || $class;
	my $self = $class->SUPER::new($mailsaobject);
	bless ($self, $class);

	$self->register_eval_rule("check_uriip");

	return $self;
}

# this is just a placeholder; in fact the results are dealt with later
sub check_uriip {
	my ($self, $permsgstatus, $rulename) = @_;
	return 0;
}

# and the eval rule itself
sub parsed_metadata {

	my ($self, $opts) = @_;
	my $scanner = $opts->{permsgstatus};

	# build a list of IPs converting the URIs
	my $reg;
	my %iplist = ();
	foreach my $uri ($scanner->get_uri_list()) {
		my $ip = my_uri_to_ip($uri);
		if ($ip) {
			dbg("debug: URIIP $ip for $uri");
			$iplist{$ip} = 1;
		}
	}

	# Now check if any match any defined rules.
	foreach my $rule (keys(%{$scanner->{conf}->{uriip}})) {
		my $ip = lc($scanner->{conf}->{uriip}->{$rule});
		if($iplist{$ip}) {
			dbg ("debug: URIIP hit rule: $ip");
			$scanner->got_hit($rule, "");
		}
	}
	return 1;
}

# Read the configuration file and build a hash with the IPs
sub parse_config {
	my ($self, $opts) = @_;
	my $key = $opts->{key};

	if ($key eq 'uriip') {
		if ($opts->{value} =~ /^(\S+)\s+(\S+)\s*$/) {
			my $rulename = $1;
			my $ip = $2;

			dbg("debug: URIIP: registering $rulename");
			$opts->{conf}->{uriip}->{$rulename} = $ip;
			$self->inhibit_further_callbacks();
			return 1;
		}
	}

	return 0;
}

# Taken from the one in Util.pm
sub my_uri_to_ip {

	my ($uri) = @_;

	# Javascript is not an URI.
	return if ($uri =~ /^javascript:/i);

	$uri =~ s,#.*$,,gs;                   # drop fragment
	$uri =~ s#^[a-z]+:/{0,2}##gsi;        # drop the protocol
	$uri =~ s,^[^/]*\@,,gs;               # username/passwd
	$uri =~ s,[/\?\&].*$,,gs;             # path/cgi params
	$uri =~ s,:\d+$,,gs;                  # port

	return if $uri =~ /\%/;         # skip undecoded URIs.

	# keep IPs intact
	if ($uri !~ /^\d+\.\d+\.\d+\.\d+$/) {
		# ignore invalid domains
		return unless (Mail::SpamAssassin::Util::RegistrarBoundaries::is_domain_valid($uri));
	}

	# convert to lowercase.
	$uri=lc $uri;

	# now convert the URI into an IP
	my $res   = Net::DNS::Resolver->new;
	$res->udp_timeout(5);
	$res->tcp_timeout(5);
	my $query = $res->search($uri,'A');

	if ($query) {
		foreach my $rr ($query->answer) {
			next unless $rr->type eq "A";
			return $rr->address;
		}
	} else {
		return;
	}
}

sub dbg { Mail::SpamAssassin::dbg (@_); }

1;
